import 'dart:async';

import 'package:flutter/material.dart';
import 'package:jiffy/jiffy.dart';

import 'default_date_render.dart';

class DatePicker extends StatefulWidget {
  final DateTime startDate;
  final DateTime endDate;
  final double size;
  final Function(DateTime selectedDate) onChange;
  final Function(DateTime date) dateRender;
  final Function(DateTime date) selectedDateRender;
  final Decoration boxRender;
  final Decoration selectedBoxRender;

  DatePicker({
    @required this.startDate,
    @required this.endDate,
    this.size = 75,
    this.onChange,
    this.dateRender,
    this.selectedDateRender,
    this.boxRender,
    this.selectedBoxRender,
  });

  @override
  _DatePickerState createState() => _DatePickerState();
}

class _DatePickerState extends State<DatePicker> {
  int _index = 0;
  List<DateTime> days = [];
  ScrollController _controller;

  get size => widget.size;

  get startDate => widget.startDate;

  get endDate => widget.endDate;

  get dateRender => widget.dateRender ?? (DateTime date) => DefaultDateRender(date);

  get selectedDateRender => widget.selectedDateRender ?? (DateTime date) => DefaultDateRender(date);

  get boxRender => widget.boxRender ?? _renderNormalBox();

  get selectedBoxRender => widget.selectedBoxRender ?? _renderSelectedBox();

  @override
  void initState() {
    super.initState();
    getDaysInBetween();
    _index = _jumpDays();

    _controller = new ScrollController();
    Timer(Duration(seconds: 0), () => jump());
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  void getDaysInBetween() {
    for (int i = 0; i <= endDate.difference(startDate).inDays; i++) {
      days.add(startDate.add(Duration(days: i)));
    }
  }

  int _jumpDays() {
    return Jiffy().diff(startDate, Units.DAY);
  }

  void jump() {
    if (!_controller.hasClients) return;

    int days = _jumpDays() - 1;

    _controller.jumpTo(days * size);
  }

  BorderSide _normalBorderSide() {
    return BorderSide(color: Colors.black26, width: 1);
  }

  BorderSide _selectedBorderSide() {
    return BorderSide(color: Color(0xFF4186E4), width: 5);
  }

  handleSelect(int index) {
    setState(() => _index = index);
    widget.onChange(days[index]);
  }

  Decoration _renderNormalBox() {
    return BoxDecoration(
      border: Border(
        right: _normalBorderSide(),
        top: _normalBorderSide(),
        bottom: _normalBorderSide(),
      ),
    );
  }

  Decoration _renderSelectedBox() {
    return BoxDecoration(
      color: Color(0xFF4186E4).withOpacity(0.12),
      border: Border(
        right: _normalBorderSide(),
        top: _normalBorderSide(),
        bottom: _selectedBorderSide(),
      ),
    );
  }

  Widget _buildDate(int index) {
    return GestureDetector(
      onTap: () => handleSelect(index),
      child: Container(
        padding: EdgeInsets.all(8),
        width: size,
        decoration: _index == index ? selectedBoxRender : boxRender,
        child: _index == index ? selectedDateRender(days[index]) : dateRender(days[index]),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: size,
      child: ListView.builder(
        // itemExtent: width,
        controller: _controller,
        itemCount: days.length,
        scrollDirection: Axis.horizontal,
        itemBuilder: (BuildContext context, int index) => _buildDate(index),
      ),
    );
  }
}
